## mainWindow.py - Contains the UI code needed for system-config-nfs
## Copyright (C) 2002, 2003 Red Hat, Inc.
## Copyright (C) 2002, 2003 Brent Fox <bfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import nfsBackend
import gtk
import gobject
import string
import os

import propertiesWindow

##
## I18N
##
from rhpl.translate import _, N_
import rhpl.translate as translate
translate.textdomain ("system-config-nfs")

##
## Icon for windows
##

iconPixbuf = None      
try:
    iconPixbuf = gtk.gdk.pixbuf_new_from_file("/usr/share/system-config-nfs/pixmaps/system-config-nfs.png")
except:
    pass

class mainWindow:
    def destroy(self, *args):
        gtk.mainquit()
    
    def __init__(self):
        self.mainWindow = gtk.Window()

        self.mainWindow.set_title(_("NFS Server Configuration"))
        self.mainWindow.set_icon(iconPixbuf)
        self.mainWindow.set_position(gtk.WIN_POS_CENTER)
        self.mainWindow.connect("delete-event", self.destroy)
        self.toplevelVBox = gtk.VBox(gtk.FALSE)
        self.menuBar = gtk.MenuBar()
        self.nameTag = _("NFS")
        self.commentTag = _("Create, modify, and delete nfs shares")

#        [0 dir, 1 hosts, 2 read/write text, 3 nfsDataObject]

        self.exportsStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING,
                                          gobject.TYPE_PYOBJECT)

        self.exportsView = gtk.TreeView(self.exportsStore)
        self.exportsView.columns_autosize()
        self.exportsViewSW = gtk.ScrolledWindow()
        self.exportsViewSW.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.exportsViewSW.set_shadow_type(gtk.SHADOW_IN)
        self.exportsViewSW.add(self.exportsView)

        self.col = gtk.TreeViewColumn(_("Directory"), gtk.CellRendererText(), text=0)        
#        self.col.set_spacing(235)
        self.exportsView.append_column(self.col)
        self.col = gtk.TreeViewColumn(_("Hosts"), gtk.CellRendererText(), text=1)        
        self.col.set_spacing(235)
        self.exportsView.append_column(self.col)
        self.col = gtk.TreeViewColumn(_("Permissions"), gtk.CellRendererText(), text=2)        
        self.exportsView.append_column(self.col)

        accel_group = gtk.AccelGroup()
        item_fac = gtk.ItemFactory(gtk.MenuBar, "<main>", accel_group)

        self.mainWindow.add_accel_group(accel_group)
        item_fac.create_items([
            ((_('/_File')),                   None,    None, 0, '<Branch>'),
            ((_('/_File/_Add Share')),    None,    self.on_add_button_clicked, 6, '<StockItem>', gtk.STOCK_ADD),
            ((_('/_File/_Properties')),       None,    self.on_properties_button_clicked, 6, '<StockItem>', gtk.STOCK_PROPERTIES),
            ((_('/_File/_Delete')),           None,    self.on_delete_button_clicked, 6, '<StockItem>', gtk.STOCK_DELETE),
            ((_('/_File/_Quit')),             None,    self.destroy, 6, '<StockItem>', gtk.STOCK_QUIT),
            ((_('/_Help')),                     None,    None, 0, '<Branch>'),
            ((_('/_Help/_Contents')),             None,  self.on_help_button_clicked, 1, '<StockItem>', gtk.STOCK_HELP),
            ((_('/_Help/_About')),              None,    self.on_about_clicked, 2, ''),
            ])
        self.menuBar = item_fac.get_widget('<main>')

        self.actionMenu = self.menuBar.get_children()[0].get_submenu()        
        self.propertiesMenu = self.actionMenu.get_children()[1]
        self.deleteMenu = self.actionMenu.get_children()[2]

        self.toolbar = gtk.Toolbar()

        #Have to do some ugly hacks here to set the mnemonic on gtk stock buttons
        button = self.toolbar.insert_stock('gtk-add', _("Add an NFS share"), None,
                                           self.on_add_button_clicked, None, 0)
        image, label = button.get_children()[0].get_children()
        label.set_text(_("_Add"))
        label.set_use_underline(gtk.TRUE)

        self.propertiesButton = self.toolbar.insert_stock('gtk-properties',
                                                          _("Edit the properties of the selected directory"),
                                                          None, self.on_properties_button_clicked, None, 1)
        image, label = self.propertiesButton.get_children()[0].get_children()
        label.set_text(_("_Properties"))
        label.set_use_underline(gtk.TRUE)
        
        self.deleteButton = self.toolbar.insert_stock('gtk-delete', _("Delete the selected directory"),
                                                      None, self.on_delete_button_clicked, None, 2)
        image, label = self.deleteButton.get_children()[0].get_children()
        label.set_text(_("_Delete"))
        label.set_use_underline(gtk.TRUE)

        self.toolbar.insert_stock('gtk-help', _("View help"), None, self.on_help_button_clicked,
                                  None, 3)

        self.propertiesButton.set_sensitive(gtk.FALSE)
        self.deleteButton.set_sensitive(gtk.FALSE)

        self.propWindow = propertiesWindow.propertiesWindow(self, self.exportsStore, self.exportsView)

        #-------------------Packing--------------------#
        self.toplevelVBox.pack_start(self.menuBar, gtk.FALSE)        
        self.toplevelVBox.pack_start(self.toolbar, gtk.FALSE)
        self.toplevelVBox.pack_start(self.exportsViewSW, gtk.TRUE)
        self.mainWindow.add(self.toplevelVBox)
        self.mainWindow.set_size_request(600, 500)

        self.selectedRow = -1
        self.changed = gtk.FALSE
        
        self.exports = nfsBackend.NfsBackend()
        self.populateExportsList()

        self.createHandler = None
        self.modifyHandler = None

#        self.exportsList.columns_autosize()
        self.mainWindow.show_all()

        self.exportsView.get_selection().connect("changed", self.on_exportsList_select_row)
        self.exportsView.get_selection().unselect_all()

#        self.exportsList.sort()
        gtk.mainloop()

    def populateExportsList(self):
        entries = self.exports.getEntries()
        for entry in entries:
            #Iterate through the entries and populate the exportsStore
            iter = self.exportsStore.append()
            self.exportsStore.set_value(iter, 0, entry.directory)
            self.exportsStore.set_value(iter, 1, entry.hosts)
            if entry.permissions == "ro":
                self.exportsStore.set_value(iter, 2, (_("Read")))
            elif entry.permissions == "rw":
                self.exportsStore.set_value(iter, 2, (_("Read/Write")))
            self.exportsStore.set_value(iter, 3, entry)

    def processData(self):
        #processData extracts the data from the list store and stores it in a list
        #We pass th at list to the backend to be written to a file
        nfsDataObjectList = []
        iter = self.exportsStore.get_iter_first()
        if iter == None:
            return nfsDataObjectList

        while iter:
            nfsDataObject = self.exportsStore.get_value(iter, 3)
            nfsDataObjectList.append(nfsDataObject)
            iter = self.exportsStore.iter_next(iter)
            
        return nfsDataObjectList

    #--------Event handlers for mainWindow-----#
    def on_help_button_clicked(self, *args):
        help_page = "file:///usr/share/doc/system-config-nfs-" + "1.2.8" + "/index.html"
	path = "/usr/bin/htmlview"
        
        if path == None:
            dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK,
                                    (_("Help is not available.")))
            dlg.set_position(gtk.WIN_POS_CENTER)
            dlg.run()
            dlg.destroy()
            return
		
        pid = os.fork()
        if not pid:
            os.execv(path, [path, help_page])

    def on_exportsList_select_row(self, *args):
        store, iter = self.exportsView.get_selection().get_selected()
        if iter:
            self.propertiesButton.set_sensitive(gtk.TRUE)
            self.deleteButton.set_sensitive(gtk.TRUE)
            self.propertiesMenu.set_sensitive(gtk.TRUE)
            self.deleteMenu.set_sensitive(gtk.TRUE)
        else:
            self.propertiesButton.set_sensitive(gtk.FALSE)
            self.deleteButton.set_sensitive(gtk.FALSE)
            self.propertiesMenu.set_sensitive(gtk.FALSE)
            self.deleteMenu.set_sensitive(gtk.FALSE)

    def on_add_button_clicked(self, *args):
        self.changed = gtk.TRUE
        self.propWindow.new_share(_("Add NFS Share"))
        self.desensitizeWidgets()

    def on_properties_button_clicked(self, *args):
        self.changed = gtk.TRUE
        store, iter = self.exportsView.get_selection().get_selected()
        if iter != None:
            self.propWindow.edit_share(_("Edit NFS Share"), iter)
        self.desensitizeWidgets()
        
    def on_delete_button_clicked(self, *args):
        self.changed = gtk.TRUE
        store, iter = self.exportsView.get_selection().get_selected()

        if iter != None:
            self.exportsStore.remove(iter)
        self.desensitizeWidgets()
        self.apply_changes()

    def apply_changes(self, *args):
        self.changed = gtk.TRUE

        #Pass the list of data into writeFile
        self.exports.writeFile(self.processData())
        self.exports.startNfs()
        self.exports.exportFs()    

    def desensitizeWidgets(self, *args):
        self.exportsView.get_selection().unselect_all()
        self.propertiesButton.set_sensitive(gtk.FALSE)
        self.deleteButton.set_sensitive(gtk.FALSE)
        self.propertiesMenu.set_sensitive(gtk.FALSE)
        self.deleteMenu.set_sensitive(gtk.FALSE)        

    def on_about_clicked(self, action, widget):
        dlg = gtk.MessageDialog (None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK,
                                 _("NFS Server Configuration Tool\n Copyright (c) 2002 Red Hat, Inc.\n "
                                   "Copyright (c) 2002 Brent Fox <bfox@redhat.com>\n Copyright (c) "
                                   "2002 Tammy Fox <tfox@redhat.com>\n A graphical interface for "
                                   "configuring NFS shares"))
        dlg.set_title(_("About"))
        dlg.set_default_size(100, 100)
        dlg.set_position (gtk.WIN_POS_CENTER)
        dlg.set_icon(iconPixbuf)
        dlg.set_border_width(2)
        dlg.set_modal(gtk.TRUE)
        rc = dlg.run()
        dlg.destroy()
