#!/usr/bin/python2

## nfs-export.py - Contains the startup script for system-config-nfs
## Copyright (C) 2004 Red Hat, Inc.
## Copyright (C) 2004 Brent Fox <bfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import sys
import getopt
import os
import nfsBackend
import nfsData

##
## I18N
##
from rhpl.translate import _, N_
import rhpl.translate as translate
translate.textdomain ("system-config-nfs")

TRUE = 1
FALSE = 0


class nfsCli:
    def __init__(self):
        self.exports = nfsBackend.NfsBackend()

    def addExport(self, dir, perm, hosts):
        if dir == "":
            self.usage()
            return

        try:
            os.listdir(dir)
        except:
            print (_("%s is not a valid directory." % dir))
            return

        if perm == "":
            print (_("Please specify a permission"))
            self.usage()
            return

        if perm != "ro" and perm != "rw":
            print (_("%s is not a valid permission." % perm))
            self.usage()
            return

        if hosts == "":
            print (_("Please specify an IP address."))
            self.usage()
            return

        nfsDataObject = nfsData.nfsData()
        nfsDataObject.set_directory(dir)
        nfsDataObject.set_hosts(hosts)
        nfsDataObject.set_permissions(perm)
        
        if async != None:
            nfsDataObject.set_sync("async")
        else:
            nfsDataObject.set_sync("sync")

        self.exports.addEntry(nfsDataObject)
        self.exports.writeFile(self.exports.getEntries())
        self.exports.startNfs()
        self.exports.exportFs()    
        
    def usage(self):
        print _("""Usage: nfs-export [OPTION...]
       --help             Print out this message.
       --dir <path>       Specify directory.
       --perm <perm>      "ro" for read only.  "rw" for read/write.
       --ip <ipaddr>      IP address of the hosts allowed to connect.
       --list             List the existing NFS exports.
       --async            Reply to NFS requests before changes are written to disk.""")

    def listExports(self):
        self.exports.printEntries()


def useCliMode(dir, perm, hosts):
    nfsCli = nfsCli(dir, perm, hosts)


if os.getuid() > 0 or os.geteuid() > 0:
    print _("You must be root to run nfs-export.")
    import sys
    sys.exit(0)

dir = ""
perm = ""
hosts = ""
async = None

try:
    args, opts = getopt.getopt(sys.argv[1:], "dph:la", ["help", "dir=", "perm=", "ip=", "list", "async"])
except getopt.GetoptError, exc:
    print exc.msg
    nfsCli().usage()
    sys.exit(2)

for (arg, value) in args:
    if arg in ("-h", "--help"):
        nfsCli().usage()
        sys.exit(1)
    if arg in ("-l", "--list"):
        nfsCli().listExports()
        sys.exit(1)
    if arg in ("-d", "--dir"):
        dir = value
    if arg in ("-p", "--perm"):
        perm = value
    if arg in ("-i", "--ip"):
        hosts = value
    if arg in ("-a", "--async"):
        async = value

nfsCli().addExport(dir, perm, hosts)
