## propertiesWindow.py - Contains the UI for the system-config-nfs share properties window
## Copyright (C) 2002, 2003 Red Hat, Inc.
## Copyright (C) 2002, 2003 Brent Fox <bfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk
import gobject
import mainWindow
import nfsData
import string
import os

##
## I18N
##
import gettext
gettext.bindtextdomain ("system-config-nfs", "/usr/share/locale")
gettext.textdomain ("system-config-nfs")
_=gettext.gettext


class propertiesWindow:
    dirEntry = gtk.Entry()
    hostEntry = gtk.Entry()
    readRadio = gtk.RadioButton(None, (_("Read-only")))
    writeRadio = gtk.RadioButton(readRadio, (_("Read / Write")))

    secureCheckButton = gtk.CheckButton(_("Allow connections from ports 1024 and higher"))
    lockCheckButton = gtk.CheckButton(_("Allow insecure file locking"))
    subtreeCheckButton = gtk.CheckButton(_("Disable subtree checking"))
    syncCheckButton = gtk.CheckButton(_("Sync write operations on request"))
    forceSyncCheckButton = gtk.CheckButton(_("Force sync of write operations immediately"))

    rootSquashCheckButton = gtk.CheckButton(_("Treat remote root user as local root"))
    userSquashCheckButton = gtk.CheckButton(_("Treat all client users as anonymous users"))
    uidCheckButton = gtk.CheckButton(_("Specify local user ID for anonymous users"))
    uidEntry = gtk.Entry()
    gidCheckButton = gtk.CheckButton(_("Specify local group ID for anonymous users"))
    gidEntry = gtk.Entry()
        
    def __init__(self, parent, store, view):
        self.parent = parent
        self.exportsStore = store
        self.exportsView = view
        self.bb = None

        self.dialog = gtk.Window()
        self.dialog.connect("delete-event", self.on_cancel_button_clicked)
        self.dialog.set_position(gtk.WIN_POS_CENTER)
        self.dialog.set_icon(mainWindow.iconPixbuf)
        self.dialog.set_modal(gtk.TRUE)        
        self.dialog.set_modal(gtk.TRUE)

        self.vbox = gtk.VBox()
        self.dialog.add(self.vbox)

        self.bb = gtk.HButtonBox()
        self.bb.set_layout(gtk.BUTTONBOX_END)
        self.bb.set_spacing(10)
        self.bb.set_border_width(10)
        self.okButton = gtk.Button(stock='gtk-ok')

        cancelButton = gtk.Button(stock='gtk-cancel')
        cancelButton.connect('clicked', self.on_cancel_button_clicked)
        self.bb.pack_start(cancelButton)
        self.bb.pack_start(self.okButton)

        self.notebook = gtk.Notebook()
        self.notebook.append_page(self.basic_page(), gtk.Label(_("Basic")))
        self.notebook.append_page(self.general_page(), gtk.Label(_("General Options")))
        self.notebook.append_page(self.user_page(), gtk.Label(_("User Access")))

        self.vbox.pack_start(self.notebook)
        self.vbox.pack_start(self.bb, gtk.FALSE)
        
    def basic_page(self):
        pageVBox = gtk.VBox(gtk.FALSE, 5)
        pageVBox.set_border_width(5)
        
        self.browseButton = gtk.Button(_("Browse..."))
        self.browseButton.connect('clicked', self.on_browseButton_clicked)
        hbox = gtk.HBox()
        hbox.set_spacing(10)
        hbox.pack_start(self.dirEntry)
        hbox.pack_start(self.browseButton)

        table = gtk.Table(2,2)
        table.set_row_spacings(5)
        table.set_col_spacings(5)
        label = gtk.Label(_("Directory:"))
        label.set_alignment(0.0, 0.5)
        table.attach(label, 0, 1, 0, 1, gtk.FILL)
        table.attach(hbox, 1, 2, 0, 1, gtk.FILL | gtk.EXPAND)
        label = gtk.Label(_("Host(s):"))
        label.set_alignment(0.0, 0.5)
        table.attach(label, 0, 1, 1, 2, gtk.FILL)
        table.attach(self.hostEntry, 1, 2, 1, 2, gtk.FILL | gtk.EXPAND)

        pageVBox.pack_start(table, gtk.FALSE, 5)
        pageVBox.pack_start(gtk.HSeparator(), gtk.FALSE, 5)
        label = gtk.Label(_("Basic permissions:"))
        label.set_alignment(0.0, 0.5)
        pageVBox.pack_start(label, gtk.FALSE, 5)
        pageVBox.pack_start(self.readRadio, gtk.FALSE)
        pageVBox.pack_start(self.writeRadio, gtk.FALSE)
        return pageVBox

    def general_page(self):
        pageVBox = gtk.VBox()
        pageVBox.set_border_width(5)

        self.syncCheckButton.connect("toggled", self.on_syncCheckButton_toggled)
        self.forceSyncCheckButton.set_sensitive(gtk.FALSE)
        
        align = gtk.Alignment(0.2, 0.5)
        align.add(self.forceSyncCheckButton)

        pageVBox.pack_start(self.secureCheckButton, gtk.FALSE)
        pageVBox.pack_start(self.lockCheckButton, gtk.FALSE)
        pageVBox.pack_start(self.subtreeCheckButton, gtk.FALSE)
        pageVBox.pack_start(self.syncCheckButton, gtk.FALSE)
        pageVBox.pack_start(align, gtk.FALSE)
        return pageVBox

    def user_page(self):
        pageVBox = gtk.VBox()
        pageVBox.set_border_width(5)
        self.rootSquashCheckButton.connect('toggled', self.on_rootSquashCheckButton_toggled)
        self.userSquashCheckButton.connect('toggled', self.on_userSquashCheckButton_toggled)
        self.uidCheckButton.connect('toggled', self.on_uidCheckButton_toggled)
        self.gidCheckButton.connect('toggled', self.on_gidCheckButton_toggled)

        pageVBox.pack_start(self.rootSquashCheckButton, gtk.FALSE)
        pageVBox.pack_start(self.userSquashCheckButton, gtk.FALSE)
        align = gtk.Alignment(0.2, 0.5)
        align.add(self.uidCheckButton)
        pageVBox.pack_start(align, gtk.FALSE)
        self.uidBox = gtk.HBox()
        self.uidBox.pack_start(gtk.Label(_("User ID:")), gtk.FALSE)
        self.uidBox.pack_start(self.uidEntry, gtk.TRUE)
        align = gtk.Alignment(0.4, 0.5)
        align.add(self.uidBox)
        pageVBox.pack_start(align, gtk.FALSE)
        align = gtk.Alignment(0.2, 0.5)
        align.add(self.gidCheckButton)
        pageVBox.pack_start(align, gtk.FALSE)
        self.gidBox = gtk.HBox()
        self.gidBox.pack_start(gtk.Label(_("Group ID:")), gtk.FALSE)
        self.gidBox.pack_start(self.gidEntry, gtk.TRUE)
        align = gtk.Alignment(0.4, 0.5)
        align.add(self.gidBox)
        pageVBox.pack_start(align, gtk.FALSE)
        self.toggle_box(gtk.FALSE)
        return pageVBox

    def on_cancel_button_clicked(self, *args):
        self.reset()
        return gtk.TRUE

    def on_browseButton_clicked(self, *args):
        dlg = gtk.FileSelection()
        dlg.set_filename(self.dirEntry.get_text())
        #Horrible hack to turn FileSelection into a directory selection
        dlg.get_children()[0].get_children()[3].get_children()[0].get_children()[1].hide()
        dlg.set_title(_("Select a directory"))
        
        result = dlg.run()
        
        if result == gtk.RESPONSE_OK:
            filename = dlg.get_filename()
            self.dirEntry.set_text(dlg.get_filename())
        dlg.destroy()

    def on_rootSquashCheckButton_toggled(self, *args):
        state = self.rootSquashCheckButton.get_active()
        self.userSquashCheckButton.set_sensitive(not state)

    def toggle_box(self, state):
        self.uidCheckButton.set_sensitive(state)
        self.uidBox.set_sensitive(state)
        self.gidCheckButton.set_sensitive(state)
        self.gidBox.set_sensitive(state)

    def on_userSquashCheckButton_toggled(self, *args):
        if self.userSquashCheckButton.get_active() == gtk.FALSE:
            self.toggle_box(gtk.FALSE)
            self.rootSquashCheckButton.set_sensitive(gtk.TRUE)
        else:
            state = self.userSquashCheckButton.get_active()
            self.rootSquashCheckButton.set_sensitive(not state)
            self.uidCheckButton.set_sensitive(state)
            self.uidBox.set_sensitive(self.uidCheckButton.get_active())
            self.gidCheckButton.set_sensitive(state)
            self.gidBox.set_sensitive(self.gidCheckButton.get_active())

    def on_uidCheckButton_toggled(self, *args):
        self.uidBox.set_sensitive(self.uidCheckButton.get_active())

    def on_gidCheckButton_toggled(self, *args):
        self.gidBox.set_sensitive(self.gidCheckButton.get_active())

    def on_syncCheckButton_toggled(self, *args):
        self.forceSyncCheckButton.set_sensitive(self.syncCheckButton.get_active())

    def new_share(self, title):
        #Make okButton connect to on_add_button_clicked
        self.okButtonHandler = self.okButton.connect('clicked', self.on_add_button_clicked)
        self.show_win(title)
        
    def edit_share(self, title, iter):
        #Make okButton connect to on_edit_button_clicked
        self.okButtonHandler = self.okButton.connect('clicked', self.on_edit_button_clicked, iter)
        self.show_win(title, iter)

    def on_edit_button_clicked(self, widget, iter):
        dir = self.dirEntry.get_text()
        host = self.hostEntry.get_text()
            
        if dir == "" or host == "":
            dlg = gtk.MessageDialog(self.dialog, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,
                                    (_("You must specify a directory and a host")))
            dlg.set_position(gtk.WIN_POS_CENTER)
            dlg.set_icon(mainWindow.iconPixbuf)
            dlg.set_modal(gtk.TRUE)
            dlg.run()
            dlg.destroy()
            return

        self.process_options(iter, dir, host)
        self.reset()
        self.parent.apply_changes()

    def on_add_button_clicked(self, *args):
	curhost = ""

        dir = self.dirEntry.get_text()
        host = self.hostEntry.get_text()
            
        if dir == "" or host == "":
            dlg = gtk.MessageDialog(self.dialog, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,
                                    (_("You must specify a directory and a host")))
            dlg.set_position(gtk.WIN_POS_CENTER)
            dlg.set_icon(mainWindow.iconPixbuf)
            dlg.set_modal(gtk.TRUE)
            dlg.run()
            dlg.destroy()
            return

        #Check and see if the filename is a valid directory
        try:
            os.listdir(dir)
        except:
            warn = gtk.MessageDialog(self.dialog, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,
                                (_("%s is not a valid directory." % dir)))
            warn.set_position(gtk.WIN_POS_CENTER)
            warn.set_icon(mainWindow.iconPixbuf)
            warn.set_modal(gtk.TRUE)
            warn.run()
            warn.destroy()                
            return

	hosttokens = string.split(host)
	if(len(hosttokens) > 1):
	    for curhost in hosttokens:
            	iter = self.exportsStore.append()
            	self.process_options(iter, dir, curhost)
	else:
       	    iter = self.exportsStore.append()
            self.process_options(iter, dir, host)

        self.reset()
        self.parent.apply_changes()


    def process_options(self, iter, dir, host):
        nfsDataObject = nfsData.nfsData()

        self.exportsStore.set_value(iter, 0, dir)
        self.exportsStore.set_value(iter, 1, host)

        nfsDataObject.set_directory(dir)
        nfsDataObject.set_hosts(host)
        
        if self.readRadio.get_active() == gtk.TRUE:
            self.exportsStore.set_value(iter, 2, "Read")
            nfsDataObject.set_permissions("ro")
        else:
            self.exportsStore.set_value(iter, 2, "Read/Write")
            nfsDataObject.set_permissions("rw")

        if self.secureCheckButton.get_active() == gtk.TRUE:
            nfsDataObject.set_insecure("insecure")

        if self.syncCheckButton.get_active() == gtk.TRUE:
            nfsDataObject.set_sync("sync")

            if self.forceSyncCheckButton.get_active() == gtk.TRUE:
                nfsDataObject.set_no_wdelay("no_wdelay")

        else:
            nfsDataObject.set_sync("async")

        if self.subtreeCheckButton.get_active() == gtk.TRUE:
            nfsDataObject.set_no_subtree_check("no_subtree_check")

        if self.lockCheckButton.get_active() == gtk.TRUE:
            nfsDataObject.set_insecure_locks("insecure_locks")

        if self.rootSquashCheckButton.get_active() == gtk.TRUE:
            nfsDataObject.set_no_root_squash("no_root_squash")

        if self.userSquashCheckButton.get_active() == gtk.TRUE:
            nfsDataObject.set_all_squash("all_squash")

            if self.uidCheckButton.get_active() == gtk.TRUE:
                nfsDataObject.set_anonuid(self.uidEntry.get_text())

            if self.gidCheckButton.get_active() == gtk.TRUE:
                nfsDataObject.set_anongid(self.gidEntry.get_text())

        self.exportsStore.set_value(iter, 3, nfsDataObject)                

    def show_win(self, title, iter=None):
        self.dialog.set_title(title)
        self.syncCheckButton.set_active(gtk.TRUE)

        if iter:
            self.dirEntry.set_text(self.exportsStore.get_value(iter, 0))
            self.hostEntry.set_text(self.exportsStore.get_value(iter, 1))


            nfsDataObject = self.exportsStore.get_value(iter, 3)

            if nfsDataObject.get_permissions() == 'ro':
                self.readRadio.set_active(gtk.TRUE)
            else:
                self.writeRadio.set_active(gtk.TRUE)

            if nfsDataObject.get_insecure():
                self.secureCheckButton.set_active(gtk.TRUE)

            if nfsDataObject.get_sync():
                if nfsDataObject.get_sync() == 'sync':
                    #If the token is sync, the check the button
                    self.syncCheckButton.set_active(gtk.TRUE)
                elif nfsDataObject.get_sync() == 'async':
                    #If the token is async, uncheck it
                    self.syncCheckButton.set_active(gtk.FALSE)
            else:
                #If no token at all, then sync is the default
                self.syncCheckButton.set_active(gtk.TRUE)                
                
            if nfsDataObject.get_no_wdelay():
                self.forceSyncCheckButton.set_active(gtk.TRUE)

            if nfsDataObject.get_no_subtree_check():
                self.subtreeCheckButton.set_active(gtk.TRUE)

            if nfsDataObject.get_insecure_locks():
                self.lockCheckButton.set_active(gtk.TRUE)

            if nfsDataObject.get_no_root_squash():
                self.rootSquashCheckButton.set_active(gtk.TRUE)

            if nfsDataObject.get_all_squash():
                self.userSquashCheckButton.set_active(gtk.TRUE)

                if nfsDataObject.get_anonuid():
                    self.uidCheckButton.set_active(gtk.TRUE)
                    self.uidEntry.set_text(nfsDataObject.get_anonuid())

                if nfsDataObject.get_anongid():
                    self.gidCheckButton.set_active(gtk.TRUE)
                    self.gidEntry.set_text(nfsDataObject.get_anongid())

        self.dialog.show_all()
        self.dirEntry.grab_focus()

    def reset(self):
        self.notebook.set_current_page(0)
        self.dirEntry.set_text("")
        self.hostEntry.set_text("")
        self.readRadio.set_active(gtk.TRUE)

        self.secureCheckButton.set_active(gtk.FALSE)
        self.lockCheckButton.set_active(gtk.FALSE)
        self.subtreeCheckButton.set_active(gtk.FALSE)
        self.syncCheckButton.set_active(gtk.FALSE)
        self.forceSyncCheckButton.set_active(gtk.FALSE)

        self.rootSquashCheckButton.set_active(gtk.FALSE)
        self.userSquashCheckButton.set_active(gtk.FALSE)
        self.uidCheckButton.set_active(gtk.FALSE)
        self.uidEntry.set_text("")
        self.gidCheckButton.set_active(gtk.FALSE)
        self.gidEntry.set_text("")

        #Remove event handler from the button because it will
        #be reassigned when the window is launched again
        self.okButton.disconnect(self.okButtonHandler)
        self.dialog.hide()

